/*******************************************************************************
 * Copyright (C) 2019 Moxa Inc. All rights reserved.
 * SPDX-License-Identifier: Apache-2.0
 *
 * UART Sample Application
 *
 * Date          Author            Comment
 * 2019-01-22    Wanhan Hsieh      Created it.
 ******************************************************************************/

/**
    \file uart.c
    \copyright 2019 Moxa Inc. All rights reserved.
    \brief <b>UART Sample</b>
    \date 2019-01-22
    \author Wanhan Hsieh
    \version V1.0
    \details
    <pre>
    <b>Introduction:</b>
    This is UART sample code.
    <b>Example:</b>
    1. Using default: ./uart
    2. Setting port, mode and baudrate: ./uart -p1 -m1 -b9600
    <b>Default:</b>
    Default baudrate = 115200
    Default mode = 0
    Default port = 0
    <b>Help:</b>
    </pre>
    \verbatim
    moxa@Moxa:~$ sudo ./uart -h
    UART sample program.

    Usage: ./uart [OPTIONS]

    Options:
            -b       UART baudrate. Default baudrate = 115200
            -m       UART mode. Default mode = 0
            -p       UART port. Default port = 0
    \endverbatim
    \b Library: \n
    \link UART UART APIs \endlink
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <iothinx/iothinxio.h>

#define BUF_LEN     4096

/// \include uart.c
int main(int argc, char **const argv)
{
    int32_t rc;
    char buf[BUF_LEN];
    int fd;
    uint32_t uart_slot = 0;
    uint32_t uart_port = UART_PORT_1;
    uint32_t uart_mode = UART_MODE_RS232;
    uint32_t uart_baudrate = 115200;

    while (-1 != (rc = getopt(argc, argv, "b:hm:p:")))
    {
        switch (rc)
        {
        case 'b':
            uart_baudrate = atoi(optarg);
            break;
        case 'p':
            uart_port = atoi(optarg);
            break;
        case 'm':
            uart_mode = atoi(optarg);
            break;
        case 'h':
        default:
            printf("UART sample program.\n\n");
            printf("Usage: ./uart [OPTIONS]\n\n");
            printf("Options:\n");
            printf("\t%-8s UART baudrate. Default baudrate = %d\n", "-b", uart_baudrate);
            printf("\t%-8s UART mode. Default mode = %d\n", "-m", uart_mode);
            printf("\t%-8s UART port. Default port = %d\n", "-p", uart_port);
            printf("\n");
            return 0;
        }
    }

    printf("UART slot = %lu\n", uart_slot);
    printf("UART port = %lu\n", uart_port);
    printf("UART mode = %lu\n", uart_mode);
    printf("UART baudrate = %lu\n", uart_baudrate);

    rc = ioThinx_Uart_Open(uart_slot, uart_port, uart_mode, uart_baudrate, &fd);
    if (rc != IOTHINX_ERR_OK)
    {
        printf("ioThinx_Uart_Open() = %d\n", rc);
        return -1;
    }
    printf("fd = %d\n", fd);
    printf("Start UART echo.\n");
    while(1)
    {
        rc = read(fd, buf, BUF_LEN);
        if(rc > 0)
        {
            rc = write(fd, buf, rc);
            if (strncmp(buf, "quit", 4) == 0)
            {
                break;
            }
        }
    }
    printf("Stop UART echo.\n");
    close(fd);

    return 0;
}
